@@ -31,15 +31,19 @@ from pikepdf import (
 
 
 class DependencyError(Exception):
-    pass
+    "A third party dependency is needed to extract images of this type."
 
 
 class UnsupportedImageTypeError(Exception):
-    pass
+    "This image is formatted in a way pikepdf does not supported."
 
 
 class NotExtractableError(Exception):
-    pass
+    "Indicates that an image cannot be directly extracted."
+
+
+class InvalidPdfImageError(Exception):
+    "This image is not valid according to the PDF 1.7 specification."
 
 
 def array_str(value):
@@ -69,7 +73,7 @@ def dict_or_array_dict(value):
     if isinstance(value, Dictionary):
         return [value.as_dict()]
     if isinstance(value, Array):
-        return [v.as_list() for v in value]
+        return [v for v in value.as_list()]
     raise NotImplementedError(value)
 
 
@@ -90,7 +94,7 @@ class PdfImageBase(ABC):
 
     @abstractmethod
     def _metadata(self, name, type_, default):
-        pass
+        raise NotImplementedError()
 
     @property
     def width(self):
@@ -155,12 +159,12 @@ class PdfImageBase(ABC):
     @property
     @abstractmethod
     def is_inline(self):
-        pass
+        raise NotImplementedError()
 
     @property
     @abstractmethod
     def icc(self):
-        pass
+        raise NotImplementedError()
 
     @property
     def indexed(self):
@@ -172,6 +176,21 @@ class PdfImageBase(ABC):
         """Size of image as (width, height)"""
         return self.width, self.height
 
+    def _approx_mode_from_icc(self):
+        if self.indexed:
+            icc_profile = self._colorspaces[1][1]
+        else:
+            icc_profile = self._colorspaces[1]
+        icc_profile_nchannels = int(icc_profile['/N'])
+
+        if icc_profile_nchannels == 1:
+            return 'L'
+
+        # Multiple channels, need to open the profile and look
+        mode_from_xcolor_space = {'RGB ': 'RGB', 'CMYK': 'CMYK'}
+        xcolor_space = self.icc.profile.xcolor_space
+        return mode_from_xcolor_space.get(xcolor_space, '')
+
     @property
     def mode(self):
         """``PIL.Image.mode`` equivalent for this image, where possible
@@ -194,18 +213,15 @@ class PdfImageBase(ABC):
                 m = 'CMYK'
             elif self.colorspace == '/ICCBased':
                 try:
-                    icc_profile = self._colorspaces[1]
-                    icc_profile_nchannels = int(icc_profile['/N'])
-                    if icc_profile_nchannels == 1:
-                        m = 'L'
-                    elif icc_profile_nchannels == 3:
-                        m = 'RGB'
-                    elif icc_profile_nchannels == 4:
-                        m = 'CMYK'
-                except (ValueError, TypeError):
-                    pass
+                    m = self._approx_mode_from_icc()
+                except (ValueError, TypeError) as e:
+                    raise NotImplementedError(
+                        "Not sure how to handle PDF image of this type"
+                    ) from e
         if m == '':
-            raise NotImplementedError("Not sure how to handle PDF image of this type")
+            raise NotImplementedError(
+                "Not sure how to handle PDF image of this type"
+            ) from None
         return m
 
     @property
@@ -252,17 +268,12 @@ class PdfImageBase(ABC):
         elif base == '/DeviceCMYK':
             base = 'CMYK'
         elif base == '/ICCBased':
-            if iccobj['/N'] == 3:
-                base = 'RGB'
-            elif iccobj['/N'] == 4:
-                base = 'CMYK'
-            elif iccobj['/N'] == 1:
-                base = 'L'
+            base = self._approx_mode_from_icc()
         return base, lookup
 
     @abstractmethod
     def as_pil_image(self):
-        pass
+        raise NotImplementedError()
 
     @staticmethod
     def _unstack_compression(buffer, filters):
@@ -280,10 +291,7 @@ class PdfImageBase(ABC):
         """
         data = memoryview(buffer)
         while len(filters) > 1 and filters[0] == '/FlateDecode':
-            try:
-                data = decompress(data)
-            except ZlibError as e:
-                raise UnsupportedImageTypeError() from e
+            data = decompress(data)
             filters = filters[1:]
         return data, filters
 
@@ -383,7 +391,14 @@ class PdfImage(PdfImageBase):
             iccstream = self._iccstream
             iccbuffer = iccstream.get_stream_buffer()
             iccbytesio = BytesIO(iccbuffer)
-            self._icc = ImageCms.ImageCmsProfile(iccbytesio)
+            try:
+                self._icc = ImageCms.ImageCmsProfile(iccbytesio)
+            except OSError as e:
+                if str(e) == 'cannot open profile from string':
+                    # ICC profile is corrupt
+                    raise UnsupportedImageTypeError(
+                        "ICC profile corrupt or not readable"
+                    ) from e
         return self._icc
 
     def _extract_direct(self, *, stream):
@@ -455,7 +470,10 @@ class PdfImage(PdfImageBase):
         elif self.bits_per_component == 1:
             if self.filters and self.filters[0] == '/JBIG2Decode':
                 if not jbig2.jbig2dec_available():
-                    raise DependencyError("jbig2dec - not installed")
+                    raise DependencyError(
+                        "jbig2dec - not installed or installed version is too old "
+                        "(older than version 0.15)"
+                    )
                 jbig2_globals_obj = self.filter_decodeparms[0][1].get('/JBIG2Globals')
                 im = jbig2.extract_jbig2(self.obj, jbig2_globals_obj)
             else:
@@ -676,13 +694,14 @@ class PdfImage(PdfImageBase):
 
         return tiff_header
 
-    def show(self):
+    def show(self):  # pragma: no cover
         """Show the image however PIL wants to"""
         self.as_pil_image().show()
 
     def __repr__(self):
-        return '<pikepdf.PdfImage image mode={} size={}x{} at {}>'.format(
-            self.mode, self.width, self.height, hex(id(self))
+        return (
+            f'<pikepdf.PdfImage image mode={self.mode} '
+            f'size={self.width}x{self.height} at {hex(id(self))}>'
         )
 
     def _repr_png_(self):
@@ -698,10 +717,6 @@ class PdfJpxImage(PdfImage):
         super().__init__(obj)
         self._jpxpil = self.as_pil_image()
 
-    def __eq__(self, other):  # pylint: disable=useless-super-delegation
-        # self._jpxpil is not relevant to equality
-        return super().__eq__(other)
-
     def _extract_direct(self, *, stream):
         data, filters = self._unstack_compression(
             self.obj.get_raw_stream_buffer(), self.filters
@@ -740,8 +755,9 @@ class PdfJpxImage(PdfImage):
         return False
 
     def __repr__(self):
-        return '<pikepdf.PdfJpxImage JPEG2000 image mode={} size={}x{} at {}>'.format(
-            self.mode, self.width, self.height, hex(id(self))
+        return (
+            f'<pikepdf.PdfJpxImage JPEG2000 image mode={self.mode} '
+            f'size={self.width}x{self.height} at {hex(id(self))}>'
         )
 
 
@@ -793,12 +809,10 @@ class PdfInlineImage(PdfImageBase):
         self.pil = None
 
     def __eq__(self, other):
-        if self.obj == other.obj and (
+        return self.obj == other.obj and (
             self._data._inline_image_raw_bytes()
             == other._data._inline_image_raw_bytes()
-        ):
-            return True
-        return False
+        )
 
     @classmethod
     def _unparse_obj(cls, obj):
@@ -838,17 +852,19 @@ class PdfInlineImage(PdfImageBase):
         return True
 
     @property
-    def icc(self):
-        raise NotImplementedError("Inline images with ICC profiles are not supported")
+    def icc(self):  # pragma: no cover
+        raise InvalidPdfImageError(
+            "Inline images with ICC profiles are not supported in the PDF specification"
+        )
 
     def __repr__(self):
-        mode = '?'
         try:
             mode = self.mode
-        except Exception:
-            pass
-        return '<pikepdf.PdfInlineImage image mode={} size={}x{} at {}>'.format(
-            mode, self.width, self.height, hex(id(self))
+        except NotImplementedError:
+            mode = '?'
+        return (
+            f'<pikepdf.PdfInlineImage image mode={mode} '
+            f'size={self.width}x{self.height} at {hex(id(self))}>'
         )
 
     def as_pil_image(self):
